<xsl:stylesheet xmlns:xsl = "http://www.w3.org/1999/XSL/Transform" version = "1.0" > 
<xsl:output method = "html" /> 


<!-- the template that processes single TR -->
<xsl:template name = "TRtempl"> 
	<xsl:param name="ltype"/>
	<TR>
	<xsl:for-each select = "./TD" >
		<xsl:variable name="pos" select="position()"/>
		<xsl:variable name="dd" select="/doc/DataDescription/DD[$pos]"/>
		<xsl:variable name="cols" select="count(/doc/DataDescription/DD)"/>
		<TD>
		<!-- defining TD class -->
		<xsl:choose>
			<xsl:when test="$ltype='rh'">
				<xsl:attribute name="class">report-header</xsl:attribute>
			</xsl:when>
			<xsl:when test="$ltype='ch'">
				<xsl:attribute name="class">report-header</xsl:attribute>
			</xsl:when>
			<xsl:otherwise>
				<xsl:attribute name="class">report-item</xsl:attribute>
			</xsl:otherwise>
		</xsl:choose>
		<!-- handle width -->
		<xsl:if test="$dd/@Width">
			<xsl:attribute name="width"><xsl:value-of select="$dd/@Width"/>%</xsl:attribute>
		</xsl:if>
		<!-- handle align (only when drawing data)-->
		<xsl:if test="$dd/@Align">
			<xsl:attribute name="align"><xsl:value-of select="$dd/@Align"/></xsl:attribute>
		</xsl:if>
		<!-- element's align attribute overrides description's attribute -->
		<xsl:if test="@Align">
			<xsl:attribute name="align"><xsl:value-of select="@Align"/></xsl:attribute>
		</xsl:if>
		
		<!-- handle spanning -->
		<xsl:if test="@Span">
			<xsl:choose>
				<!-- handle full-row spanning -->
				<xsl:when test="@Span='*'">
					<xsl:attribute name="colspan"><xsl:value-of select="$cols"/></xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="colspan"><xsl:value-of select="@Span"/></xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
		<!-- call template StyleTempl to form "style" attribute contents -->
		<xsl:attribute name="style">
			<xsl:call-template name="StyleTempl"><xsl:with-param name="dd" select="$dd"/></xsl:call-template>
		</xsl:attribute>
		<!-- output data depending on styles -->
		<xsl:choose>
			<xsl:when test="$ltype='rh'">
				<xsl:attribute name="class">report-text-title</xsl:attribute>
				<xsl:attribute name="align">left</xsl:attribute>
				<xsl:value-of select="." />
			</xsl:when>
			<xsl:when test="$ltype='ch'">
				<xsl:value-of select="." />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="." />
			</xsl:otherwise>
		</xsl:choose>

		</TD>

	</xsl:for-each> 
	</TR>
</xsl:template> 

<xsl:template name="StyleTempl">
<xsl:param name="dd"/>
		<!-- handle font name -->
		<xsl:if test="($dd/@FontName)">
			<xsl:text>FONT-FAMILY:</xsl:text><xsl:value-of select="$dd/@FontName"/><xsl:text>;</xsl:text>
		</xsl:if> 
	<!-- handle font size -->
		<xsl:if test="$dd/@FontSize">
			<xsl:text>FONT-SIZE:</xsl:text><xsl:value-of select="$dd/@FontSize"/><xsl:text>px;</xsl:text>
		</xsl:if>
		<!-- handle font flags -->
		<xsl:if test="$dd/@FontBold">
			<xsl:text>FONT-WEIGHT:bold;</xsl:text>
		</xsl:if>
		<xsl:if test="$dd/@FontItalic">
			<xsl:text>FONT-STYLE:italic;</xsl:text>
		</xsl:if>
		<xsl:if test="$dd/@FontUnderline">
			<xsl:text>TEXT-DECORATION:underline;</xsl:text>
		</xsl:if>
</xsl:template>

<xsl:template match = "/" > 

<TABLE class="report-report" width="100%" border="1" style="border-collapse:collapse;">

<xsl:for-each select="/doc/ReportHeader/TR">
	<xsl:call-template name="TRtempl"><xsl:with-param name="ltype">rh</xsl:with-param></xsl:call-template>
</xsl:for-each>

<xsl:for-each select="/doc/ColumnHeader/TR">
	<xsl:call-template name="TRtempl"><xsl:with-param name="ltype">ch</xsl:with-param></xsl:call-template>
</xsl:for-each>

<xsl:for-each select="/doc/Data/TR">
	<xsl:call-template name="TRtempl"><xsl:with-param name="ltype">da</xsl:with-param></xsl:call-template>
</xsl:for-each>

<xsl:for-each select="/doc/ReportFooter/TR">
	<xsl:call-template name="TRtempl"><xsl:with-param name="ltype">ch</xsl:with-param></xsl:call-template>
</xsl:for-each>

</TABLE>
</xsl:template> 
</xsl:stylesheet> 